/*
	Compile with:
	gcc -o diomon diomon.c -lkstat
	Run with:
	./diomon  3
	or similar.
*/

#include <stdio.h>
#include <kstat.h>

#define DELAY 3
#define LOOP 1

int main (int argc, char **argv)
{

   kstat_ctl_t *kc;
   kstat_t *ksp;
   u_int delay, i = 0;
   int loop;

   struct ufs_directio_kstats
   {
      u_int logical_reads;
      u_int phys_reads;
      u_int hole_reads;
      u_int nread;
      u_int logical_writes;
      u_int phys_writes;
      u_int nwritten;
      u_int nflushes;
   } ks1, ks2;

   if (!(kc = kstat_open()))
   {
      perror("Error on kstat_open()");
      exit(1);
   }

   if (!(ksp = kstat_lookup(kc,
      "ufs directio",
      -1,
      "UFS DirectIO Stats")))
   {
      printf("Cannot find \
[ufs directio:*:UFS DirectIO Stats]\n");
      exit(2);
   }

   if (argc > 1 && (atoi(argv[1]) > 0))
   {
      delay = atoi(argv[1]);
   } else
   delay = DELAY;

   if (argc > 2 && (atoi(argv[2]) > 0))
      loop = atoi(argv[2]);
   else if (argc > 1)
      loop = -1;
   else
      loop = LOOP;

   if (kstat_read(kc, ksp, &ks1) == -1)
   {
      perror("Error in  kstat_read()");
      exit(3);
   }
   while (i != loop)
   {
      if (!(i % 25))
         printf("%10s%10s%10s%10s%10s%10s%10s%10s\n",
            @ "lreads", "preads", "lwrites", "pwrites",
            "Kread", "Kwrite", "holeread", "nflush");
      sleep(delay);
      if (kstat_read(kc, ksp, &ks2) == -1)
      {
         perror("Error in kstat_read()");
         exit(3);
      }
      printf("%10u%10u%10u%10u%10u%10u%10u%10u\n",
         ks2.logical_reads - ks1.logical_reads,
         ks2.phys_reads - ks1.phys_reads,
         ks2.logical_writes - ks1.logical_writes,
         ks2.phys_writes - ks1.phys_writes,
         (ks2.nread - ks1.nread) / 1024,
         (ks2.nwritten - ks1.nwritten)  / 1024,
         ks2.hole_reads - ks1.hole_reads,
         ks2.nflushes - ks1.nflushes);
      i++;
      ks1 = ks2;
   }
}
